/*
 * Copyright (C) 2006, Movial Corporation
 * Author: Toni Timonen <ttimonen@movial.fi>
 *
 * Contains some parts made by
 * Lauri Leukkunen <lauri.leukkunen@nokia.com>
 * Copyright (C) 2004, 2005 Nokia
 *
 *
 * Licenced under GPL, see COPYING for details.
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#define LIB_DIR        "/scratchbox/host_shared/lib/"
#define LD_PATH        LIB_DIR "ld.so"

static int endswith(char *haystack,char *needle) {
	return strcmp(&haystack[strlen(haystack)-strlen(needle)],needle)==0;
}

static int oneStartswith(char **data,char *key) {
	int len=strlen(key);
	if(!data) return 0;
	char **p;
	for(p=data;*p;p++) if (strncmp(*p,key,len)==0) return 1;
	return 0;
}

static int elemCount(char **elems) {
	int count=0;
	char **p=elems;
	while(*p) {p++; count++;}
	return count;
}

// Returns the number of splittable elements in s
static int substrCount(char *s,char sep) {
	int count=1;
	char *p;
	if(!s)
		return 0;
	for(p=s;*p;p++) if(*p==sep) count++;
	return count;
}

char *envs[]={"LD_LIBRARY_PATH","LD_RUN_PATH"};

char **extra_args(char *fname,char **args) {
	char **ret=NULL;
	if (endswith(fname,"ld") && !oneStartswith(args,"-rpath")) {
		int nOfElems=0;
		int i=0,j=0;
		char *p;
		// Figure out list size.
		for(;i<2;i++)
			nOfElems+=substrCount(getenv(envs[i]),':');
		ret=(char**)calloc(nOfElems+1,sizeof(char*));
		ret[nOfElems]=NULL;

		// Fill the list
		for(i=0,j=0;i<2;i++) {
			for(p=getenv(envs[i]);p;p=strchr(p,':')) {
				int len;
				p++;
				if(!strchr(p,':'))  //This is the last entry
					len=strlen(p);
				else
					len=strchr(p,':')-p;
				ret[j]=(char*)calloc(len+3,sizeof(char));
				strcpy(ret[j],"-L");
				strncpy(ret[j++]+2,p,len);
			}
		}

	}
	if(!ret) {
		ret=(char**)calloc(1,sizeof(char*));
	}
	return ret;
}

int main(int argc, char **argv) {
	char *binaryname, *exe, **my_argv;
	char **extra;
	char **p;
	int i = 0, j;

	binaryname = basename(strdup(argv[1]));


	exe = argv[1];

	extra=extra_args(binaryname,&argv[2]);

	my_argv = (char**)calloc(4 + (argc - 1) + 1+elemCount(extra), sizeof (char *));
	my_argv[i++] = LD_PATH;
	my_argv[i++] = "--library-path";
	my_argv[i++] = LIB_DIR;
	my_argv[i++] = exe;

	for(p=extra;*p;p++) my_argv[i++]=*p;


	for (j = 2; j < argc;  ++j)
		my_argv[i++] = argv[j];

	execv(my_argv[0], my_argv);

	fprintf(stderr, "sb_alien (running %s): %s\n", exe, strerror(errno));
	return -11;
}

